/*
 * An opener for a file employing a rewritable memory area.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <systypes/fcntl.h>

#include "block_file_accessor.h"
#include "block_file_opener.h"

BlockFileOpener::~BlockFileOpener()
{
}

/* Return a file identifier for the given 'path'. */

long BlockFileOpener::get_fileid(const char *path, flags_t flags, fileid_t *fileid)
{
    /* Obtain any registered identifier for the path. */

    *fileid = _get_fileid(path, false);

    if (*fileid != FILEID_INVALID)
        return L4_EOK;

    /* Test for a valid file or an attempt to create a file. */

    FILE *fp = fopen(path, "r");

    if (fp == NULL)
    {
        if (!(flags & O_CREAT))
            return -L4_ENOENT;
    }
    else
        fclose(fp);

    *fileid = _get_fileid(path, true);
    return L4_EOK;
}

/* Return a new accessor for 'fileid'. */

long BlockFileOpener::make_accessor(const char *path, flags_t flags,
                                    fileid_t fileid, Accessor **accessor)
{
    FILE *fp = fopen(path, "r");

    if (fp == NULL)
    {
        if (flags & O_CREAT)
            *accessor = new BlockFileAccessor(fileid);
        else
            return -L4_ENOENT;
    }
    else
        *accessor = new BlockFileAccessor(fp, fileid);

    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
