/*
 * Interprocess communication abstractions.
 *
 * Copyright (C) 2018, 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/utcb.h>

#include "direct.h"
#include "util_ipc.h"



/* Direct operations. */

/* Reserve the given number of slots from zero for incoming capabilities. */

long ipc_expect_capabilities(int number)
{
  return _expect_capabilities(l4_utcb_br(), number);
}

/* Reserve a slot for an incoming capability. */

long ipc_expect_capability(int item)
{
  return _expect_capability(l4_utcb_br(), item);
}

/* Export a capability at the given position in the message. */

void ipc_export_capability(l4_msgtag_t tag, int item, l4_cap_idx_t ref)
{
  _export_capability(tag, l4_utcb_mr(), item, ref);
}

/* Import the capability at the given item position, updating the buffer
   registers for future capabilities. */

long ipc_import_capability(l4_msgtag_t tag, int item, l4_cap_idx_t *ref)
{
  return _import_capability(tag, l4_utcb_br(), l4_utcb_mr(), item, ref);
}

/* Import a dataspace, mapping it to an address, updating the buffer registers
   for future capabilities. */

long ipc_import_dataspace(l4_msgtag_t tag, int item, l4re_ds_t *mem, l4_addr_t *addr)
{
  return _import_dataspace(tag, l4_utcb_br(), l4_utcb_mr(), item, mem, addr);
}
